import time

from dopy.manager import DoManager, DoError
from prudentia.domain import Box
from prudentia.factory import FactoryProvider, FactoryCli
from prudentia.simple import SimpleProvider
from prudentia.utils.provisioning import create_user
from prudentia.utils import io


class DigitalOceanCli(FactoryCli):
    def __init__(self):
        FactoryCli.__init__(self)
        self.prompt = '(Prudentia > DigitalOcean) '
        self.provider = DigitalOceanProvider()


class DigitalOceanProvider(FactoryProvider):
    NAME = 'digital-ocean'

    DEFAULT_IMAGE_SLUG = "ubuntu-14-04-x64"  # Ubuntu latest LTS 64bit
    DEFAULT_SIZE_SLUG = "1gb"
    DEFAULT_REGION_SLUG = "ams3"

    def __init__(self):
        super(DigitalOceanProvider, self).__init__(self.NAME, DOGeneral, DOExt)
        if (not self.env.initialized) or (not self.env.general) or (not self.env.general.api_token):
            g = self._input_general_env_conf()
        else:
            g = self.env.general
        self.manager = DoManager(None, g.api_token, api_version=2)

    def _input_general_env_conf(self):
        print '\nThe DigitalOcean API (v2) token is not configured.'
        print 'If you don\'t have it please visit ' \
              'https://cloud.digitalocean.com/settings/applications and generate one.'
        api_token = io.input_value('api token')
        do_general = DOGeneral(api_token)
        self.env.set_general(do_general)
        return do_general

    def define_box(self):
        ip = None
        ext = DOExt()

        if io.input_yes_no('register an existing droplet'):
            droplet_id = io.input_value('droplet id')
            droplet_info = self.manager.show_droplet(droplet_id)
            ext.id = droplet_id
            name = droplet_info['name']
            created = droplet_info['created_at']
            print '\nFound droplet \'{0}\' (created {1})'.format(name, created)
            ip = io.xstr(droplet_info['ip_address'])
            print 'IP: %s' % ip
            ext.image = droplet_info['image']['id']
            print 'Image: %s' % ext.image
            ext.size = droplet_info['size']['slug']
            print 'Size: %s' % ext.size
            ext.region = droplet_info['region']['slug']
            print 'Region: %s\n' % ext.region

            playbook = io.input_path('playbook path')
            hostname = self.fetch_box_hosts(playbook)
            user = io.input_value('remote user', self.active_user)
        else:
            playbook = io.input_path('playbook path')
            hostname = self.fetch_box_hosts(playbook)
            name = io.input_value('box name', self.suggest_name(hostname))
            user = io.input_value('remote user', self.active_user)

            all_images = self.manager.all_images()
            print '\nAvailable images: \n%s' % self._print_object_id_name(all_images)
            default_image = next((img for img in all_images
                                  if self.DEFAULT_IMAGE_SLUG in img['slug']), None)
            if default_image:
                image_desc = '{0} - {1} {2}'.format(
                    default_image['id'],
                    default_image['distribution'],
                    default_image['name']
                )
                ext.image = io.input_value('image', default_image['id'], image_desc)
            else:
                ext.image = io.input_value('image')

            all_sizes = self.manager.sizes()
            sizes_slug = [o['slug'] for o in all_sizes]
            print '\nAvailable sizes: \n%s' % '\n'.join(sizes_slug)
            ext.size = io.input_choice('size', self.DEFAULT_SIZE_SLUG, choices=sizes_slug)

            all_regions = self.manager.all_regions()
            regions_slug = [o['slug'] for o in all_regions]
            print '\nAvailable regions: \n%s' % '\n'.join(regions_slug)
            ext.region = io.input_choice('region', self.DEFAULT_REGION_SLUG, choices=regions_slug)

            ext.keys = self._input_ssh_keys()

        return Box(name, playbook, hostname, ip, user, extra=ext)

    @staticmethod
    def _print_object_id_name(objs):
        def _dist_if_exists(prop):
            return prop['distribution'] + ' ' if 'distribution' in prop else ''

        return '\n'.join([str(o['id']) + ' -> ' + _dist_if_exists(o) + o['name'] for o in objs])

    def redefine_box(self, previous_box):
        playbook = io.input_path('playbook path', previous_box.playbook)
        hostname = self.fetch_box_hosts(playbook)
        ip = previous_box.ip
        user = io.input_value('remote user', previous_box.remote_user)

        if not previous_box.extra.id:
            ext = DOExt()
            ext.id = previous_box.extra.id
            all_images = self.manager.all_images()
            print '\nAvailable images: \n%s' % self._print_object_id_name(all_images)
            prev_image = next((img for img in all_images
                               if previous_box.extra.image == img['id']), None)
            if prev_image:
                prev_image_desc = '{0} - {1} {2}'.format(
                    prev_image['id'],
                    prev_image['distribution'],
                    prev_image['name']
                )
                ext.image = io.input_value('image', prev_image['id'], prev_image_desc)
            else:
                ext.image = io.input_value('image')

            all_sizes = self.manager.sizes()
            sizes_slug = [o['slug'] for o in all_sizes]
            print '\nAvailable sizes: \n%s' % '\n'.join(sizes_slug)
            ext.size = io.input_value('size', previous_box.extra.size)

            all_regions = self.manager.all_regions()
            regions_slug = [o['slug'] for o in all_regions]
            print '\nAvailable regions: \n%s' % '\n'.join(regions_slug)
            ext.region = io.input_value('region', previous_box.extra.region)

            ext.keys = self._input_ssh_keys(previous_box.extra.keys)
        else:
            ext = previous_box.extra

        return Box(previous_box.name, playbook, hostname, ip, user, extra=ext)

    def add_box(self, box):
        if not box.ip:
            self.create(box)
        SimpleProvider.add_box(self, box)

    def create(self, box):
        ext = box.extra
        if not ext.id:
            if not ext.keys:
                print '\nNo valid keys are defined.'
                print 'Please run `reconfigure {0}` to provide them.'.format(box.name)
                return False
            print '\nCreating instance \'{0}\' ...'.format(box.name)
            droplet = self.manager.new_droplet(
                name=box.name, size_id=ext.size,
                image_id=ext.image, region_id=ext.region,
                ssh_key_ids=ext.keys.split(',')
            )
            box.extra.id = droplet['id']
            box.ip = self._wait_to_be_active(box.extra.id)
        else:
            info = self.manager.show_droplet(ext.id)
            print 'Droplet {0} already exists - status: {1}.'.format(ext.id, info['status'])
        create_user(box, self.loader)

    def start(self, box):
        box_id = box.extra.id
        print 'Starting droplet %s ...' % box_id
        self.manager.power_on_droplet(box_id)
        self._wait_to_be_active(box_id)

    def stop(self, box):
        box_id = box.extra.id
        print 'Stopping droplet %s ...' % box_id
        self.manager.power_off_droplet(box_id)

    def destroy(self, box):
        if io.input_yes_no('destroy the droplet \'{0}\''.format(box.name)):
            box_id = box.extra.id
            box.ip = None
            box.extra.id = None
            print 'Destroying droplet %s ...' % box_id
            self.manager.destroy_droplet(box_id, scrub_data=True)

    def rebuild(self, box):
        ext = box.extra
        print 'Rebuilding droplet %s ...' % ext.id
        self.manager.rebuild_droplet(ext.id, ext.image)
        self._wait_to_be_active(ext.id)
        create_user(box, self.loader)

    def status(self, box):
        print self.manager.show_droplet(box.extra.id)['status']

    def _wait_to_be_active(self, droplet_id, wait_timeout=300):
        end_time = time.time() + wait_timeout
        while time.time() < end_time:
            print 'Waiting for droplet %s to be active ...' % droplet_id
            time.sleep(min(20, end_time - time.time()))

            droplet = self.manager.show_droplet(droplet_id)
            if droplet['status'] == 'active':
                droplet_ip_address = droplet['ip_address']
                if not droplet_ip_address:
                    raise DoError('No ip is found.', droplet_id)
                print '\nDroplet %s is now active with ip %s\n' % (droplet_id, droplet_ip_address)
                time.sleep(10)  # Wait for some network latency ...
                return droplet_ip_address
        raise DoError('Wait for droplet running timeout', droplet_id)

    def _input_ssh_keys(self, previous=None):
        all_keys = self.manager.all_ssh_keys()
        print '\nAvailable keys: \n%s' % self._print_object_id_name(all_keys)
        if not previous:
            default_keys = ','.join([str(k['id']) for k in all_keys])
        else:
            default_keys = previous
        return io.input_value('keys', default_keys)


class DOGeneral(object):
    def __init__(self, api_token):
        self.api_token = api_token

    def __repr__(self):
        return 'DOGeneral[api_token: %s]' % self.api_token

    def to_json(self):
        return {'api_token': self.api_token}

    @staticmethod
    def from_json(json):
        return DOGeneral(json.get('api_token'))


class DOExt(object):
    id = None
    image = None
    size = None
    keys = None
    region = None

    def __repr__(self):
        return 'DOExt[id: %s, image: %s, size: %s, keys: %s, region: %s]' % (
            self.id, self.image, self.size, self.keys, self.region)

    def to_json(self):
        return {'id': self.id, 'image': self.image, 'size': self.size,
                'keys': self.keys, 'region': self.region}

    @staticmethod
    def from_json(json):
        ext = DOExt()
        ext.id = json['id']
        ext.image = json['image']
        ext.size = json['size']
        ext.keys = json['keys']
        ext.region = json['region']
        return ext



import logging
import os
import pwd
import random
from abc import ABCMeta, abstractmethod
from cmd import Cmd

from ansible.parsing.dataloader import DataLoader
from ansible.playbook import Playbook
from prudentia.domain import Environment
from prudentia.utils import io
from prudentia.utils import provisioning


class SimpleCli(Cmd):
    provider = None  # Set by his children

    def cmdloop(self, intro=None):
        try:
            Cmd.cmdloop(self, intro)
        except Exception as ex:
            io.track_error('something bad happened', ex)

    def complete_box_names(self, text, line, begidx, endidx):
        completions = ['']
        tokens = line.split(' ')
        action = tokens[0]
        box_input = tokens[1]
        if len(tokens) <= 2:
            # boxes completion
            # when matching part of a box names returns only the possible endings
            il = len(box_input)
            completions = [text + b.name[il:] for b in self.provider.boxes() if b.name.startswith(box_input)]
        else:
            if action == 'provision':
                # tags completion
                if not text:
                    completions = self.provider.tags[box_input][:]
                else:
                    completions = [t for t in self.provider.tags[box_input] if t.startswith(text)]
                current_tags = tokens[2:]
                completions = [c for c in completions if c not in current_tags]
        return completions

    @staticmethod
    def help_register():
        print "Registers a new box.\n"

    def do_register(self, line):
        self.provider.register()

    @staticmethod
    def help_reconfigure():
        print "Reconfigures an existing box.\n"

    def complete_reconfigure(self, text, line, begidx, endidx):
        return self.complete_box_names(text, line, begidx, endidx)

    def do_reconfigure(self, line):
        box = self.provider.get_box(line)
        if box:
            self.provider.reconfigure(box)

    @staticmethod
    def help_provision():
        print "Starts and provisions the box, it accepts as optional argument a playbook tag.\n"

    def complete_provision(self, text, line, begidx, endidx):
        return self.complete_box_names(text, line, begidx, endidx)

    def do_provision(self, line):
        tokens = line.split(' ')
        box = self.provider.get_box(tokens[0])
        if box:
            self.provider.provision(box, tokens[1:])

    @staticmethod
    def help_unregister():
        print "Unregisters an existing box.\n"

    def complete_unregister(self, text, line, begidx, endidx):
        return self.complete_box_names(text, line, begidx, endidx)

    def do_unregister(self, line):
        box = self.provider.get_box(line)
        if box:
            self.provider.unregister(box)

    @staticmethod
    def help_set():
        print "Sets the value of an Ansible extra variable. " \
              "During provisioning it will forcibly override the one defined in any playbook.\n"

    def do_set(self, line):
        try:
            first_space_idx = line.index(' ')
            name = line[:first_space_idx].strip()
            value = line[first_space_idx:].strip()
            self.provider.set_var(name, value)
        except ValueError:
            logging.exception('Error in setting variable for the current provider.')
            print 'Please provide the name of the variable followed by its value.\n'

    @staticmethod
    def help_envset():
        print "Sets the value of an environment variable.\n"

    def do_envset(self, line):
        try:
            first_space_idx = line.index(' ')
            name = line[:first_space_idx].strip()
            value = line[first_space_idx:].strip()
            os.environ[name] = value
        except ValueError:
            logging.exception('Error in setting variable for the current provider.')
            print 'Please provide the name of the variable followed by its value.\n'

    @staticmethod
    def help_unset():
        print "Unsets an existing Ansible extra variable. " \
              "If this action is invoked without parameter it will show " \
              "the current set of variables.\n"

    def do_unset(self, line):
        self.provider.unset_var(line)

    @staticmethod
    def help_list():
        print "Shows a list of current boxes.\n"

    def do_list(self, line):
        boxes = self.provider.boxes()
        if not len(boxes):
            print 'No box has been registered yet.\n'
        else:
            for b in boxes:
                print b

    @staticmethod
    def help_decrypt():
        print "Provides the password that will be used to decrypt Ansible vault files. " \
              "For more information visit http://docs.ansible.com/playbooks_vault.html.\n"

    def do_decrypt(self, line):
        self.provider.set_vault_password()

    @staticmethod
    def help_vars():
        print "Loads Ansible extra vars from a .yml or .json file " \
              "(they will override existing ones).\n"

    def do_vars(self, line):
        self.provider.load_vars(line.strip())

    @staticmethod
    def help_verbose():
        print "Sets Ansible verbosity. Allowed values are " \
              "between 0 (only task status) and 4 (full connection info).\n"

    def do_verbose(self, line):
        self.provider.verbose(line.strip())

    @staticmethod
    def help_facts():
        print 'Gathers and shows useful information about the box. ' \
              'Accepts optional parameter to filter shown properties.'

    def complete_facts(self, text, line, begidx, endidx):
        return self.complete_box_names(text, line, begidx, endidx)

    def do_facts(self, line):
        tokens = line.split(' ')
        box = self.provider.get_box(tokens[0])
        if box:
            self.provider.facts(box, *tokens[1:])

    @staticmethod
    def do_EOF(line):
        print "\n"
        return True

    def emptyline(self, *args, **kwargs):
        return ""


class SimpleProvider(object):
    __metaclass__ = ABCMeta

    def __init__(self, name, general_type=None, box_extra_type=None):
        self.env = Environment(name, general_type, box_extra_type)
        self.loader = DataLoader()
        self.provisioned = False
        self.tags = {}
        self.extra_vars = {'prudentia_dir': io.prudentia_python_dir()}
        self.load_tags()
        self.active_user = pwd.getpwuid(os.geteuid())[0]

    def boxes(self):
        return self.env.boxes.values()

    def get_box(self, box_name):
        b = self.env.get(box_name)
        if not b:
            print 'The box \'%s\' you entered does not exists.\n\n' \
                  'After typing the command press Tab for box suggestions.\n' % box_name
            return None
        else:
            return b

    def _show_current_vars(self):
        print 'Current set variables:\n%s\n' % '\n'.join(
            [n + ' -> ' + str(v) for n, v in self.extra_vars.iteritems()]
        )

    def set_var(self, var, value):
        if var in self.extra_vars:
            print 'NOTICE: Variable \'{0}\' is already set to this value: \'{1}\' ' \
                  'and it will be overwritten.'.format(var, self.extra_vars[var])
        self.extra_vars[var] = value
        if provisioning.VERBOSITY > 0:
            print "Set \'{0}\' -> {1}\n".format(var, value)

    def unset_var(self, var):
        if not var:
            print 'Please provide a valid variable name to unset.\n'
            self._show_current_vars()
        elif var not in self.extra_vars:
            print 'WARNING: Variable \'{0}\' is NOT present so cannot be unset.\n'.format(var)
            self._show_current_vars()
        else:
            self.extra_vars.pop(var, None)
            print "Unset \'{0}\'\n".format(var)

    def set_vault_password(self):
        vault_pwd = io.input_value('Ansible vault password', hidden=True)
        self.loader.set_vault_password(vault_pwd)

    def load_vars(self, vars_file):
        if not vars_file:
            vars_file = io.input_path('path of the variables file')
        vars_dict = self.loader.load_from_file(vars_file)
        for key, value in vars_dict.iteritems():
            self.set_var(key, value)

    def add_box(self, box):
        self.env.add(box)
        self.load_tags(box)

    def load_tags(self, box=None):
        for b in [box] if box else self.boxes():
            if not os.path.exists(b.playbook):
                print 'WARNING: Box \'{0}\' points to a NON existing playbook. ' \
                      'Please `reconfigure` or `unregister` the box.\n'.format(b.name)
            else:
                plays = Playbook.load(b.playbook, loader=self.loader).get_plays()
                all_tags = set()
                for p in plays:
                    for block in p.compile():
                        for task in block.block:
                            all_tags.update(task.tags)
                self.tags[b.name] = list(all_tags)

    def remove_box(self, box):
        if box.name in self.tags:
            self.tags.pop(box.name)
        return self.env.remove(box)

    def register(self):
        try:
            box = self.define_box()
            if box:
                self.add_box(box)
                print "\nBox %s added." % box
        except Exception as ex:
            io.track_error('cannot add box', ex)

    @abstractmethod
    def define_box(self):
        pass

    def reconfigure(self, previous_box):
        try:
            box = self.redefine_box(previous_box)
            if box:
                self.remove_box(previous_box)
                self.add_box(box)
                print "\nBox %s reconfigured." % box
        except Exception as ex:
            io.track_error('cannot reconfigure box', ex)

    @abstractmethod
    def redefine_box(self, previous_box):
        pass

    def unregister(self, box):
        self.remove_box(box)
        print "\nBox %s removed.\n" % box.name

    def fetch_box_hosts(self, playbook):
        ds = self.loader.load_from_file(playbook)
        if ds:
            return ds[0]['hosts']  # a playbook is an array of plays we take the first one

    def suggest_name(self, hostname):
        if hostname not in self.env.boxes:
            return hostname
        else:
            return hostname + '-' + str(random.randint(0, 100))

    def provision(self, box, tags):
        only_tags = None
        if len(tags) > 0:
            only_tags = tags

        self.provisioned = provisioning.run_playbook(
            playbook_file=box.playbook,
            inventory_file=provisioning.generate_inventory(box),
            loader=self.loader,
            remote_user=box.get_remote_user(),
            remote_pass=box.get_remote_pwd(),
            transport=box.get_transport(),
            extra_vars=self.extra_vars,
            only_tags=only_tags
        )

    @staticmethod
    def verbose(value):
        if value:
            try:
                iv = int(value)
            except ValueError:
                iv = -1
            if 0 <= iv <= 4:
                provisioning.VERBOSITY = iv
            else:
                print 'Verbosity value \'{0}\' not allowed, should be a number between 0 and 4.'.format(value)
        else:
            print 'Current verbosity: {0}'.format(provisioning.VERBOSITY)

    def facts(self, box, regex='*'):
        return provisioning.gather_facts(box, regex, self.loader)



# (c) 2015, Jonathan Davila <jdavila(at)ansible.com>
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
# USAGE: {{ lookup('hashi_vault', 'secret=secret/hello token=c975b780-d1be-8016-866b-01d0f9b688a5 url=http://myvault:8200')}}
#
# You can skip setting the url if you set the VAULT_ADDR environment variable
# or if you want it to default to localhost:8200
#
# NOTE: Due to a current limitation in the HVAC library there won't
# necessarily be an error if a bad endpoint is specified.
#
# Requires hvac library. Install with pip.
#

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import os

from ansible.errors import *

ANSIBLE_HASHI_VAULT_ADDR = 'http://127.0.0.1:8200'

if os.getenv('VAULT_ADDR') is not None:
    ANSIBLE_HASHI_VAULT_ADDR = os.environ['VAULT_ADDR']


class HashiVaultLookup:
    def __init__(self, **kwargs):
        try:
            import hvac
        except ImportError:
            AnsibleError("Please pip install hvac to use this module")

        self.url = kwargs.pop('url')
        self.secret = kwargs.pop('secret')
        self.token = kwargs.pop('token')

        self.client = hvac.Client(url=self.url, token=self.token)

        if self.client.is_authenticated():
            pass
        else:
            raise AnsibleError("Invalid Hashicorp Vault Token Specified")

    def get(self):
        data = self.client.read(self.secret)
        if data is None:
            raise AnsibleError("The secret %s doesn't seem to exist" % self.secret)
        else:
            return data['data']['value']


class LookupModule(object):
    def __init__(self, basedir=None, **kwargs):
        pass

    def run(self, terms, inject=None, **kwargs):
        vault_args = terms.split(' ')
        vault_dict = {}
        ret = []

        for param in vault_args:
            key, value = param.split('|')
            vault_dict[key] = value

        vault_conn = HashiVaultLookup(**vault_dict)
        value = vault_conn.get()
        ret.append(value)

        if 'write_to_file' in vault_dict.keys():
            text_file = open(vault_dict['write_to_file'], "w")
            text_file.write(value)
            text_file.close()

        return ret



import argparse
import os
from os import path
import sys

from . import __version__ as prudentia_ver
from ansible import __version__ as ansible_ver


# Setting Ansible config file environment variable as first thing
cwd = path.dirname(path.realpath(__file__))
os.environ['ANSIBLE_CONFIG'] = path.join(cwd, 'ansible.cfg')
os.environ['ANSIBLE_ROLES_PATH'] = path.join(cwd, 'roles') + os.pathsep + '/etc/ansible/roles'
os.environ['ANSIBLE_LOOKUP_PLUGINS'] = path.join(cwd, 'plugins', 'lookup') + \
                                       os.pathsep + '/usr/share/ansible_plugins/lookup_plugins'
os.environ['ANSIBLE_LIBRARY'] = path.join(cwd, 'modules')

from prudentia.digital_ocean import DigitalOceanCli
from prudentia.local import LocalCli
from prudentia.ssh import SshCli
from prudentia.vagrant import VagrantCli

Providers = {
    'local': LocalCli,
    'ssh': SshCli,
    'vagrant': VagrantCli,
    'digital-ocean': DigitalOceanCli
}


def parse(args=None):
    parser = argparse.ArgumentParser(
        prog='Prudentia',
        description='A useful Continuous Deployment toolkit.'
    )
    parser.add_argument('-v', '--version', action='version',
                        version='%(prog)s ' + prudentia_ver + ', Ansible ' + ansible_ver)
    parser.add_argument('provider', choices=Providers.keys(),
                        help='use one of the available providers')
    parser.add_argument('commands', nargs='*', default='',
                        help='optional quoted list of commands to run with the chosen provider')
    if len(sys.argv) == 1:
        parser.print_help()
        sys.exit(1)
    return parser.parse_args(args)


def run(args):
    chosen_cli = Providers[args.provider]()
    if args.commands:
        for c in args.commands:
            print "Executing: '{0}'\n".format(c)
            chosen_cli.onecmd(c)
    else:
        chosen_cli.cmdloop()
    return chosen_cli.provider.provisioned



from prudentia.domain import Box
from prudentia.simple import SimpleProvider, SimpleCli
from prudentia.utils import io


class SshCli(SimpleCli):
    def __init__(self):
        SimpleCli.__init__(self)
        self.prompt = '(Prudentia > Ssh) '
        self.provider = SshProvider()


class SshProvider(SimpleProvider):
    NAME = 'ssh'

    def __init__(self):
        super(SshProvider, self).__init__(self.NAME)

    def define_box(self):
        playbook = io.input_path('playbook path')
        hostname = self.fetch_box_hosts(playbook)
        name = io.input_value('box name', self.suggest_name(hostname))
        ip = io.input_value('instance address or inventory')
        user = io.input_value('remote user', self.active_user)
        pwd = io.input_value(
            'password for the remote user',
            default_description='ssh key',
            mandatory=False,
            hidden=True
        )
        return Box(name, playbook, hostname, ip, user, pwd)

    def redefine_box(self, previous_box):
        playbook = io.input_path('playbook path', previous_box.playbook)
        hostname = self.fetch_box_hosts(playbook)
        ip = io.input_value('instance address or inventory', previous_box.ip)
        user = io.input_value('remote user', previous_box.remote_user)
        if previous_box.remote_pwd:
            pwd = io.input_value(
                'password for the remote user',
                default_value=previous_box.remote_pwd,
                default_description='*****',
                mandatory=False,
                hidden=True
            )
        else:
            pwd = io.input_value(
                'password for the remote user',
                default_description='ssh key',
                mandatory=False,
                hidden=True
            )
        return Box(previous_box.name, playbook, hostname, ip, user, pwd)
